const DOTS_KEY = /^[\w.]+$/g;
const ESCAPE_REGEXP = /\\(\\)?/g;
const PROPERTY_REGEXP = RegExp('[^.[\\]]+' +
    '|' +
    '\\[(?:' +
    '([^"\'][^[]*)' +
    '|' +
    '(["\'])((?:(?!\\2)[^\\\\]|\\\\.)*?)\\2' +
    ')\\]' +
    '|' +
    '(?=(?:\\.|\\[\\])(?:\\.|\\[\\]|$))', 'g');
function toPath(deepKey) {
    if (DOTS_KEY.test(deepKey)) {
        return deepKey.split('.');
    }
    const result = [];
    if (deepKey[0] === '.') {
        result.push('');
    }
    const matches = deepKey.matchAll(PROPERTY_REGEXP);
    for (const match of matches) {
        let key = match[0];
        const expr = match[1];
        const quote = match[2];
        const substr = match[3];
        if (quote) {
            key = substr.replace(ESCAPE_REGEXP, '$1');
        }
        else if (expr) {
            key = expr;
        }
        result.push(key);
    }
    return result;
}

export { toPath };
